@@ -1,9 +1,8 @@
 /*
  * utility funcs
  *
- * Copyright 2005-2010 Gentoo Foundation
+ * Copyright 2005-2014 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/xsystem.c,v 1.2 2011/02/21 22:02:59 vapier Exp $
  */
 
 #include <stdlib.h>
@@ -28,8 +27,12 @@ static void xsystembash(const char *command)
 
 	default: /* parent */
 		waitpid(p, &status, 0);
-		if (WIFEXITED(status) && WEXITSTATUS(status) == 0)
-			return;
+		if (WIFEXITED(status)) {
+			if (WEXITSTATUS(status) == 0)
+				return;
+			else
+				exit(WEXITSTATUS(status));
+		}
 		/* fall through */
 
 	case -1: /* fucked */
