@@ -1,10 +1,9 @@
 /*
- * Copyright 2005-2010 Gentoo Foundation
+ * Copyright 2005-2014 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qlop.c,v 1.62 2013/04/21 04:28:10 vapier Exp $
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2010 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2014 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #ifdef APPLET_qlop
@@ -52,7 +51,6 @@ static const char * const qlop_opts_help[] = {
 	"Read emerge logfile instead of " QLOP_DEFAULT_LOGFILE,
 	COMMON_OPTS_HELP
 };
-static const char qlop_rcsid[] = "$Id: qlop.c,v 1.62 2013/04/21 04:28:10 vapier Exp $";
 #define qlop_usage(ret) usage(ret, QLOP_FLAGS, qlop_long_opts, qlop_opts_help, lookup_applet_idx("qlop"))
 
 #define QLOP_LIST    0x01
@@ -387,7 +385,8 @@ void show_current_emerge(void)
 	DIR *proc;
 	struct dirent *de;
 	pid_t pid;
-	char buf[BUFSIZE], bufstat[300];
+	static char *cmdline, *bufstat;
+	static size_t cmdline_len, bufstat_len;
 	char path[50];
 	char *p, *q;
 	unsigned long long start_time = 0;
@@ -409,17 +408,17 @@ void show_current_emerge(void)
 
 		/* portage renames the cmdline so the package name is first */
 		snprintf(path, sizeof(path), "/proc/%i/cmdline", pid);
-		if (!eat_file(path, buf, sizeof(buf)))
+		if (!eat_file(path, &cmdline, &cmdline_len))
 			continue;
 
-		if (buf[0] == '[' && (p = strchr(buf, ']')) != NULL && strstr(buf, "sandbox") != NULL) {
+		if (cmdline[0] == '[' && (p = strchr(cmdline, ']')) != NULL && strstr(cmdline, "sandbox") != NULL) {
 			*p = '\0';
-			p = buf+1;
+			p = cmdline + 1;
 			q = p + strlen(p) + 1;
 
 			/* open the stat file to figure out how long we have been running */
 			snprintf(path, sizeof(path), "/proc/%i/stat", pid);
-			if (!eat_file(path, bufstat, sizeof(bufstat)))
+			if (!eat_file(path, &bufstat, &bufstat_len))
 				continue;
 
 			/* ripped from procps/proc/readproc.c */
@@ -437,7 +436,7 @@ void show_current_emerge(void)
 				"%llu ",
 				&start_time);
 			/* get uptime */
-			if (!eat_file("/proc/uptime", bufstat, sizeof(bufstat)))
+			if (!eat_file("/proc/uptime", &bufstat, &bufstat_len))
 				continue;
 			sscanf(bufstat, "%lf", &uptime_secs);
 
